@@ -30,19 +30,18 @@ import tempfile
 import time
 import signal
 import pathlib
-
-try:
-    # Python 2
-    from urllib import url2pathname
-except ImportError:
-    # Python 3
-    from urllib.request import url2pathname
-
+import platform
+import configparser
+import warnings
+from urllib.request import url2pathname
 from copy import copy
 
 sharedir = os.path.join(sys.prefix, 'share')
 basedir = '.'
-if sys.argv[0]:
+if getattr(sys, 'frozen', False):
+    basedir = os.path.dirname(sys.executable)
+    sharedir = os.path.join(basedir, 'share')
+elif sys.argv[0]:
     execdir = os.path.dirname(os.path.realpath(sys.argv[0]))
     basedir = os.path.dirname(execdir)
     sharedir = os.path.join(basedir, 'share')
@@ -65,27 +64,24 @@ if os.name == 'nt':
     del libintl
 else:
     locale.bindtextdomain(DOMAIN, localedir)
-gettext.bindtextdomain(DOMAIN, localedir)
-gettext.textdomain(DOMAIN)
-_ = gettext.gettext
 
-APPNAME = 'pdfarranger'
-VERSION = '1.0'
+APPNAME = 'PDF Arranger'
+VERSION = '1.2.0'
 WEBSITE = 'https://github.com/jeromerobert/pdfarranger'
 LICENSE = 'GNU General Public License (GPL) Version 3.'
 
-try:
-    import gi
-    gi.require_version('Gtk', '3.0')
-    from gi.repository import Gtk
-except:
-    print('You do not have the required version of GTK+ installed.\n\n' +
+import gi
+# check that we don't need GObject.threads_init()
+gi.check_version('3.10.2')
+gi.require_version('Gtk', '3.0')
+from gi.repository import Gtk
+if Gtk.check_version(3,4,0):
+    raise Exception('You do not have the required version of GTK+ installed. ' +
           'Installed GTK+ version is ' +
           '.'.join([str(Gtk.get_major_version()),
                     str(Gtk.get_minor_version()),
-                    str(Gtk.get_micro_version())]) + '\n' +
-          'Required GTK+ version is 3.0 or higher')
-    sys.exit(1)
+                    str(Gtk.get_micro_version())]) +
+          '. Required GTK+ version is 3.4 or higher.')
 
 from gi.repository import Gdk
 from gi.repository import GObject      # for using custom signals
@@ -94,24 +90,110 @@ from gi.repository import GLib
 gi.require_version('Poppler', '0.18')
 from gi.repository import Poppler      #for the rendering of pdf pages
 import cairo
-
-try:
-    from PyPDF2 import PdfFileWriter, PdfFileReader
-except ImportError:
-    from pyPdf import PdfFileWriter, PdfFileReader
+from PyPDF2 import PdfFileWriter, PdfFileReader
 
 from .iconview import CellRendererImage
 GObject.type_register(CellRendererImage)
 
+if os.name == 'nt' and GLib.get_language_names():
+    os.environ['LANG']=GLib.get_language_names()[0]
+gettext.bindtextdomain(DOMAIN, localedir)
+gettext.textdomain(DOMAIN)
+_ = gettext.gettext
 
-class PdfArranger:
-    prefs = {
-        'window width': min(700, Gdk.Screen.get_default().get_width() / 2),
-        'window height': min(600, Gdk.Screen.get_default().get_height() - 50),
-        'initial thumbnail size': 300,
-        'initial zoom level': 0,
-    }
+def _install_workaround_bug29():
+    """ Install a workaround for https://gitlab.gnome.org/GNOME/pygobject/issues/29 """
+    try:
+        gi.check_version('3.29.2')
+    except ValueError:
+        def func(self, entries):
+            # simplified version of https://gitlab.gnome.org/GNOME/pygobject/commit/d0b219c
+            for d in entries:
+                param_type = None if len(d) < 3 else GLib.VariantType.new(d[2])
+                action = Gio.SimpleAction(name=d[0], parameter_type=param_type)
+                action.connect("activate", d[1], None)
+                self.add_action(action)
+        Gtk.ApplicationWindow.add_action_entries = func
 
+_install_workaround_bug29()
+
+
+class Config(object):
+    """ Wrap a ConfigParser object for PDFArranger """
+    @staticmethod
+    def _config_file():
+        """ Return the location of the configuration file """
+        home = os.path.expanduser("~")
+        if platform.system() == 'Darwin':
+            p = os.path.join(home, 'Library', 'Caches')
+        elif 'LOCALAPPDATA' in os.environ:
+            p = os.getenv('LOCALAPPDATA')
+        elif 'XDG_CACHE_HOME' in os.environ:
+            p = os.getenv('XDG_CACHE_HOME')
+        else:
+            p = os.path.join(home, '.cache')
+        p = os.path.join(p, DOMAIN)
+        os.makedirs(p, exist_ok=True)
+        return os.path.join(p, 'config.ini')
+
+    def __init__(self):
+        self.data = configparser.ConfigParser()
+        self.data.add_section('window')
+        self.data.read(Config._config_file())
+
+    def window_size(self):
+        ds = Gdk.Screen.get_default()
+        return self.data.getint('window', 'width', fallback=int(min(700, ds.get_width() / 2))), \
+               self.data.getint('window', 'height', fallback=int(min(600, ds.get_height() - 50)))
+
+    def set_window_size(self, size):
+        self.data.set('window', 'width', str(size[0]))
+        self.data.set('window', 'height', str(size[1]))
+
+    def maximized(self):
+        return self.data.getboolean('window', 'maximized', fallback=False)
+
+    def set_maximized(self, m):
+        self.data.set('window', 'maximized', str(m))
+
+    def zoom_level(self):
+        return self.data.getint('DEFAULT', 'zoom-level', fallback=0)
+
+    def set_zoom_level(self, l):
+        self.data.set('DEFAULT', 'zoom-level', str(l))
+
+    def save(self):
+        conffile = Config._config_file()
+        os.makedirs(os.path.dirname(conffile), exist_ok=True)
+        with open(conffile, 'w') as f:
+            self.data.write(f)
+
+
+def warn_dialog(func):
+    """ Decorator which redirect warnings module messages to a gkt MessageDialog """
+    class ShowWarning(object):
+        def __init__(self):
+            self.buffer=""
+
+        def __call__(self, message, category, filename, lineno, f=None, line=None):
+            s=warnings.formatwarning(message, category, filename, lineno, line)
+            sys.stderr.write(s+'\n')
+            self.buffer+=str(message)+'\n'
+
+    def wrapper(*args, **kwargs):
+        self = args[0]
+        backup_showwarning=warnings.showwarning
+        warnings.showwarning=ShowWarning()
+        try:
+            func(*args, **kwargs)
+            if len(warnings.showwarning.buffer) > 0:
+                self.error_message_dialog(warnings.showwarning.buffer, Gtk.MessageType.WARNING)
+        finally:
+            warnings.showwarning=backup_showwarning
+    return wrapper
+
+
+class PdfArranger(Gtk.Application):
     MODEL_ROW_INTERN = 1001
     MODEL_ROW_EXTERN = 1002
     TEXT_URI_LIST = 1003
@@ -122,14 +204,68 @@ class PdfArranger:
     TARGETS_SW = [Gtk.TargetEntry.new('text/uri-list', 0, TEXT_URI_LIST),
                   Gtk.TargetEntry.new('MODEL_ROW_EXTERN', Gtk.TargetFlags.OTHER_APP, MODEL_ROW_EXTERN)]
 
-    def __init__(self):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, application_id="com.github.jeromerobert.pdfarranger",
+                         flags=Gio.ApplicationFlags.HANDLES_OPEN|Gio.ApplicationFlags.NON_UNIQUE,
+                         **kwargs)
+
+    def do_open(self, files, n, hints):
+        """ https://lazka.github.io/pgi-docs/Gio-2.0/classes/Application.html#Gio.Application.do_open """
+        self.activate()
+        # Importing documents passed as command line arguments
+        for f in files:
+            self.add_pdf_pages(f.get_path())
+
+    def __create_actions(self):
+        # Both Gtk.ApplicationWindow and Gtk.Application are Gio.ActionMap. Some action are window
+        # related some other are application related. As pdfarrager is a single window app does not
+        # matter that much.
+        self.window.add_action_entries([
+            ('rotate', self.rotate_page, 'i'),
+            ('delete', self.clear_selected),
+            ('crop', self.crop_page_dialog),
+            ('export-selection', self.choose_export_selection_pdf_name),
+            ('reverse-order', self.reverse_order),
+            ('save', self.on_action_save),
+            ('save-as', self.on_action_save_as),
+            ('import', self.on_action_add_doc_activate),
+            ('zoom', self.zoom_change, 'i'),
+            ('quit', self.on_quit),
+        ])
+        accels = [
+            ('delete', 'Delete'),
+            ('crop', 'c'),
+            ('rotate(90)', '<Ctrl>Right'),
+            ('rotate(-90)', '<Ctrl>Left'),
+            ('save', '<Ctrl>s'),
+            ('save-as', '<Ctrl><Shift>s'),
+            ('quit', '<Ctrl>q'),
+            ('import', 'Insert'),
+            ('zoom(5)', 'plus'),
+            ('zoom(-5)', 'minus'),
+        ]
+        for a, k in accels:
+            self.set_accels_for_action("win." + a, [k])
+        # Disable actions
+        self.iv_selection_changed_event()
+        # Display accelerators in the menus
+        for o in self.uiXML.get_objects():
+            if isinstance(o, Gtk.MenuItem) and o.get_action_name() is not None:
+                an = Gio.Action.print_detailed_name(o.get_action_name(), o.get_action_target_value())
+                a = self.get_accels_for_action(an)
+                if len(a) > 0:
+                    o.get_child().set_accel(*Gtk.accelerator_parse(a[0]))
+
+    def do_activate(self):
+        """ https://lazka.github.io/pgi-docs/Gio-2.0/classes/Application.html#Gio.Application.do_activate """
+        # TODO: huge method that should be splitted
         # Create the temporary directory
         self.tmp_dir = tempfile.mkdtemp(DOMAIN)
         os.chmod(self.tmp_dir, 0o700)
 
         iconsdir = os.path.join(sharedir, 'icons')
         if not os.path.exists(iconsdir):
-            iconsdir = os.path.join(sharedir, 'data')
+            iconsdir = os.path.join(sharedir, 'data', 'icons')
         Gtk.IconTheme.get_default().append_search_path(iconsdir)
         Gtk.Window.set_default_icon_name(DOMAIN)
 
@@ -142,6 +278,7 @@ class PdfArranger:
         if not os.path.exists(ui_path):
             ui_path = '/usr/local/share/{}/{}.ui'.format(DOMAIN, DOMAIN)
 
+        self.config = Config()
         self.uiXML = Gtk.Builder()
         self.uiXML.set_translation_domain(DOMAIN)
         self.uiXML.add_from_file(ui_path)
@@ -152,8 +289,10 @@ class PdfArranger:
         self.window = self.uiXML.get_object('main_window')
         self.window.set_title(APPNAME)
         self.window.set_border_width(0)
-        self.window.set_default_size(self.prefs['window width'],
-                                     self.prefs['window height'])
+        self.window.set_application(self)
+        if self.config.maximized():
+            self.window.maximize()
+        self.window.set_default_size(*self.config.window_size())
         self.window.connect('delete_event', self.close_application)
 
         if hasattr(GLib, "unix_signal_add"):
@@ -172,10 +311,6 @@ class PdfArranger:
         self.sw.connect('button_press_event', self.sw_button_press_event)
         self.sw.connect('scroll_event', self.sw_scroll_event)
 
-        # Create an alignment to keep the thumbnails center-aligned
-        align = Gtk.Alignment.new(0.5, 0.5, 0, 0)
-        self.sw.add_with_viewport(align)
-
         # Create ListStore model and IconView
         self.model = Gtk.ListStore(str,         # 0.Text descriptor
                                    GObject.TYPE_PYOBJECT,
@@ -193,33 +328,15 @@ class PdfArranger:
                                    float,       # 12.Page height
                                    float)       # 13.Resampling factor
 
-        self.zoom_set(self.prefs['initial zoom level'])
-        self.iv_col_width = self.prefs['initial thumbnail size']
+        self.zoom_set(self.config.zoom_level())
 
         self.iconview = Gtk.IconView(self.model)
         self.iconview.clear()
-        self.iconview.set_item_width(-1) #self.iv_col_width + 12)
+        self.iconview.set_item_width(-1)
 
         self.cellthmb = CellRendererImage()
         self.iconview.pack_start(self.cellthmb, False)
         self.iconview.set_cell_data_func(self.cellthmb, self.set_cellrenderer_data, None)
-#        self.iconview.add_attribute(self.cellthmb, 'image', 1)
-#        self.iconview.add_attribute(self.cellthmb, 'scale', 4)
-#        self.iconview.add_attribute(self.cellthmb, 'rotation', 6)
-#        self.iconview.add_attribute(self.cellthmb, 'cropL', 7)
-#        self.iconview.add_attribute(self.cellthmb, 'cropR', 8)
-#        self.iconview.add_attribute(self.cellthmb, 'cropT', 9)
-#        self.iconview.add_attribute(self.cellthmb, 'cropB', 10)
-#        self.iconview.add_attribute(self.cellthmb, 'width', 11)
-#        self.iconview.add_attribute(self.cellthmb, 'height', 12)
-#        self.iconview.add_attribute(self.cellthmb, 'resample', 13)
-
-#        self.celltxt = Gtk.CellRendererText()
-#        self.celltxt.set_property('width', self.iv_col_width)
-#        self.celltxt.set_property('wrap-width', self.iv_col_width)
-#        self.celltxt.set_property('alignment', Pango.Alignment.CENTER)
-#        self.iconview.pack_start(self.celltxt, False)
-#        self.iconview.add_attribute(self.celltxt, 'text', 0)
         self.iconview.set_text_column(0)
 
         self.iconview.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
@@ -241,15 +358,14 @@ class PdfArranger:
         self.iconview.connect('button_release_event', self.iv_button_release_event)
         self.iconview.connect('selection_changed', self.iv_selection_changed_event)
 
-        align.add(self.iconview)
+        self.sw.add_with_viewport(self.iconview)
 
         # Progress bar
         self.progress_bar = self.uiXML.get_object('progressbar')
         self.progress_bar_timeout_id = 0
 
         # Define window callback function and show window
-        self.window.connect('size_allocate', self.on_window_size_request)        # resize
-        self.window.connect('key_press_event', self.on_keypress_event ) # keypress
+        self.window.connect('check_resize', self.on_window_size_request)
         self.window.show_all()
         self.progress_bar.hide()
 
@@ -267,24 +383,8 @@ class PdfArranger:
         self.iconview.override_background_color(Gtk.StateFlags.PRELIGHT,
                                                 color_prelight)
 
-        # Creating the popup menu
-        self.popup = Gtk.Menu()
-        labels = (_('_Rotate Right'), _('Rotate _Left'), _('C_rop...'),
-                  _('_Delete'), _('Re_verse Order'), _('_Export selection...'))
-        cbs = (self.rotate_page_right, self.rotate_page_left,
-               self.crop_page_dialog, self.clear_selected,
-               self.reverse_order,
-               self.choose_export_selection_pdf_name)
-        for label, cb in zip(labels, cbs):
-           popup_item = Gtk.MenuItem.new_with_mnemonic(label)
-           popup_item.connect('activate', cb)
-           popup_item.show()
-           self.popup.append(popup_item)
-           if cb == self.reverse_order:
-                self.reverse_order_popup_menu_item = popup_item
-
-        # Application menu
-        self.reverse_order_app_menu_item = self.uiXML.get_object('imagemenuitem_reverse_order')
+        self.popup = self.uiXML.get_object('popup_menu')
+        self.popup.attach_to_widget(self.window, None)
 
         # Initializing variables
         self.export_directory = os.path.expanduser('~')
@@ -304,9 +404,7 @@ class PdfArranger:
 
         self.set_unsaved(False)
         self.export_file = None
-        # Importing documents passed as command line arguments
-        for filename in sys.argv[1:]:
-            self.add_pdf_pages(filename)
+        self.__create_actions()
 
     def set_cellrenderer_data(self, column, cell, model, iter, data=None):
         cell.set_property('image', model.get_value(iter,1))
@@ -324,9 +422,7 @@ class PdfArranger:
         if self.rendering_thread:
             self.rendering_thread.quit = True
             self.rendering_thread.join()
-        #FIXME: the resample=1. factor has to be dynamic when lazy rendering
-        #       is implemented
-        self.rendering_thread = PDFRenderer(self.model, self.pdfqueue, 1)
+        self.rendering_thread = PDFRenderer(self.model, self.pdfqueue, 1 / self.zoom_scale)
         self.rendering_thread.connect('update_thumbnail', self.update_thumbnail)
         self.rendering_thread.start()
 
@@ -361,7 +457,7 @@ class PdfArranger:
             cnt_all += 1
             if row[1]:
                 cnt_finished += 1
-        fraction = float(cnt_finished) / float(cnt_all)
+        fraction = 1 if cnt_all == 0 else cnt_finished / cnt_all
 
         self.progress_bar.set_fraction(fraction)
         self.progress_bar.set_text(_('Rendering thumbnails... [%(i1)s/%(i2)s]')
@@ -380,14 +476,27 @@ class PdfArranger:
         row[4] = self.zoom_scale
         row[1] = thumbnail
 
-    def on_window_size_request(self, window, event):
+    def on_window_size_request(self, window):
         """Main Window resize - workaround for autosetting of
            iconview cols no."""
-
-        #add 12 because of: http://bugzilla.gnome.org/show_bug.cgi?id=570152
-        col_num = 9 * window.get_size()[0] \
-            / (10 * (self.iv_col_width + self.iconview.get_column_spacing() * 2))
-        self.iconview.set_columns(col_num)
+        if len(self.model) > 0:
+            # scale*page_width*(1-crop_left-crop_right)
+            item_width = int(max(row[4]*row[11]*(1.-row[7]-row[8]) \
+                             for row in self.model))
+            # FIXME: those are magic number found with my current GTK style. This might not be portable.
+            min_col_spacing=19
+            min_margin=14
+            iw_width = window.get_size()[0]
+            # 2 * min_margin + col_num * item_width + min_col_spacing * (col_num-1) = iw_width
+            # min_margin+margin = min_col_spacing+col_spacing = (iw_width - col_num * item_width) / (col_num+1)
+            col_num = (iw_width - 2 * min_margin - min_col_spacing) // (item_width + min_col_spacing)
+            spacing = (iw_width - col_num * item_width) // (col_num + 1)
+            if col_num==0:
+                col_num=1
+                spacing=min_margin
+            self.iconview.set_columns(col_num)
+            self.iconview.set_column_spacing(spacing - min_col_spacing)
+            self.iconview.set_margin_left(spacing - min_margin)
 
     def update_geometry(self, iter):
         """Recomputes the width and height of the rotated page and saves
@@ -409,39 +518,8 @@ class PdfArranger:
 
         self.model.set(iter, 11, w1, 12, h1)
 
-    def reset_iv_width(self, renderer=None):
-        """Reconfigures the width of the iconview columns"""
-
-        if not self.model.get_iter_first(): #just checking if model is empty
-            return
-
-        max_w = 10 + int(max(row[4]*row[11]*(1.-row[7]-row[8]) \
-                             for row in self.model))
-        if max_w != self.iv_col_width:
-            self.iv_col_width = max_w
-#            self.celltxt.set_property('width', self.iv_col_width)
-#            self.celltxt.set_property('wrap-width', self.iv_col_width)
-            self.iconview.set_item_width(-1) #self.iv_col_width + 12) #-1)
-            self.on_window_size_request(self.window, None)
-
-    def on_keypress_event(self, widget, event):
-        """Keypress events in Main Window"""
-
-        #print ("Key %s (%d) was pressed" % (Gdk.keyval_name(event.keyval), event.keyval))
-        if event.state & Gdk.ModifierType.CONTROL_MASK:
-            if event.keyval == 65363:  # Key Right
-                print('rotating right')
-                self.rotate_page(90)
-            elif event.keyval == 65361:  # Key Left
-                print('rotating left')
-                self.rotate_page(-90)
-            elif event.keyval == 115: # Key 's'
-                self.on_action_save('')
-
-        if event.keyval == 65535:   # Delete keystroke
-            self.clear_selected()
-        elif event.keyval == 65379:  # Key Insert
-            self.on_action_add_doc_activate('')
+    def on_quit(self, action, param, unknown):
+        self.close_application()
 
     def close_application(self, widget=None, event=None, data=None):
         """Termination"""
@@ -453,14 +531,13 @@ class PdfArranger:
 
         # Release Poppler.Document instances to unlock all temporay files
         self.pdfqueue = []
-
+        self.config.set_window_size(self.window.get_size())
+        self.config.set_maximized(self.window.is_maximized())
+        self.config.set_zoom_level(self.zoom_level)
+        self.config.save()
         if os.path.isdir(self.tmp_dir):
             shutil.rmtree(self.tmp_dir)
-        if Gtk.main_level():
-            Gtk.main_quit()
-        else:
-            sys.exit(0)
-        return False
+        self.quit()
 
     def add_pdf_pages(self, filename,
                             firstpage=None, lastpage=None,
@@ -512,13 +589,12 @@ class PdfArranger:
             self.update_geometry(iter)
             res = True
 
-        self.reset_iv_width()
         GObject.idle_add(self.retitle)
         if res:
             GObject.idle_add(self.render)
         return res
 
-    def choose_export_pdf_name(self, widget=None, only_selected=False):
+    def choose_export_pdf_name(self, only_selected=False):
         """Handles choosing a name for exporting """
 
         chooser = Gtk.FileChooserDialog(title=_('Export ...'),
@@ -561,15 +637,18 @@ class PdfArranger:
             all_files.add(f.filename)
         return all_files
 
-    def on_action_save(self, widget, data=None):
+    def on_action_save(self, action, param, unknown):
         try:
             if self.export_file:
                 self.save(False, self.export_file)
             else:
-                self.choose_export_pdf_name(widget)
+                self.choose_export_pdf_name()
         except Exception as e:
             self.error_message_dialog(e)
 
+    def on_action_save_as(self, action, param, unknown):
+        self.choose_export_pdf_name()
+
     def save(self, only_selected, file_out):
         """Saves to the specified file.  May throw exceptions."""
         (path, shortname) = os.path.split(file_out)
@@ -581,17 +660,19 @@ class PdfArranger:
         self.export_file = file_out
         self.set_unsaved(False)
 
-    def choose_export_selection_pdf_name(self, widget=None):
-        self.choose_export_pdf_name(widget, True)
+    def choose_export_selection_pdf_name(self, action, target, unknown):
+        self.choose_export_pdf_name(True)
 
+    @warn_dialog
     def export_to_file(self, file_out, only_selected=False):
         """Export to file"""
 
         selection = self.iconview.get_selected_items()
         pdf_output = PdfFileWriter()
         pdf_input = []
+        metadata = None
         for pdfdoc in self.pdfqueue:
-            pdfdoc_inp = PdfFileReader(open(pdfdoc.copyname, 'rb'))
+            pdfdoc_inp = PdfFileReader(open(pdfdoc.copyname, 'rb'), strict=False, overwriteWarnings=False)
             if pdfdoc_inp.getIsEncrypted():
                 try: # Workaround for lp:#355479
                     stat = pdfdoc_inp.decrypt('')
@@ -605,6 +686,9 @@ class PdfArranger:
                 #FIXME
                 #else
                 #   ask for password and decrypt file
+            if metadata is None:
+                # get the metadata of the first imported document
+                metadata = pdfdoc_inp.getDocumentInfo()
             pdf_input.append(pdfdoc_inp)
 
         for row in self.model:
@@ -645,10 +729,11 @@ class PdfArranger:
 
             pdf_output.addPage(current_page)
 
+        pdf_output.addMetadata(metadata)
         # finally, write "output" to document-output.pdf
         pdf_output.write(open(file_out, 'wb'))
 
-    def on_action_add_doc_activate(self, widget, data=None):
+    def on_action_add_doc_activate(self, action, param, unknown):
         """Import doc"""
 
         chooser = Gtk.FileChooserDialog(title=_('Import...'),
@@ -702,25 +787,24 @@ class PdfArranger:
         chooser.destroy()
         GObject.idle_add(self.retitle)
 
-    def clear_selected(self, button=None):
+    def clear_selected(self, action, parameter, unknown):
         """Removes the selected elements in the IconView"""
 
         model = self.iconview.get_model()
         selection = self.iconview.get_selected_items()
-        if selection:
-            selection.sort(reverse=True)
-            self.set_unsaved(True)
-            for path in selection:
-                iter = model.get_iter(path)
-                model.remove(iter)
-            path = selection[-1]
-            self.iconview.select_path(path)
-            if not self.iconview.path_is_selected(path):
-                if len(model) > 0:	# select the last row
-                    row = model[-1]
-                    path = row.path
-                    self.iconview.select_path(path)
-            self.iconview.grab_focus()
+        selection.sort(reverse=True)
+        self.set_unsaved(True)
+        for path in selection:
+            iter = model.get_iter(path)
+            model.remove(iter)
+        path = selection[-1]
+        self.iconview.select_path(path)
+        if not self.iconview.path_is_selected(path):
+            if len(model) > 0:	# select the last row
+                row = model[-1]
+                path = row.path
+                self.iconview.select_path(path)
+        self.iconview.grab_focus()
 
     def iv_drag_begin(self, iconview, context):
         """Sets custom icon on drag begin for multiple items selected"""
@@ -949,14 +1033,12 @@ class PdfArranger:
                 self.popup.popup(None, None, None, None, event.button, time)
             return 1
 
-    def iv_selection_changed_event(self, user_data):
+    def iv_selection_changed_event(self, user_data=None):
         selection = self.iconview.get_selected_items()
-
-        # Set sensitivity of the 'Reverse Order' action according to the
-        # selection.
-        sensitive = self.reverse_order_available(selection)
-        self.reverse_order_app_menu_item.set_sensitive(sensitive)
-        self.reverse_order_popup_menu_item.set_sensitive(sensitive)
+        ne = len(selection) > 0
+        for a, e in [ ("reverse-order", self.reverse_order_available(selection)),
+            ("delete", ne), ("crop", ne), ("rotate", ne) ]:
+            self.window.lookup_action(a).set_enabled(e)
 
     def sw_dnd_received_data(self, scrolledwindow, context, x, y,
                              selection_data, target_id, etime):
@@ -1007,28 +1089,20 @@ class PdfArranger:
                 zoom_delta = -1
 
             if zoom_delta != 0:
-                self.zoom_change(zoom_delta)
+                self.zoom_set(self.zoom_level + zoom_delta)
                 return 1
 
     def zoom_set(self, level):
         """Sets the zoom level"""
-        self.zoom_level = max(min(level, 20), -6)
+        self.zoom_level = max(min(level, 40), -10)
         self.zoom_scale = 0.2 * (1.1 ** self.zoom_level)
         for row in self.model:
             row[4] = self.zoom_scale
-        self.reset_iv_width()
+        GObject.idle_add(self.render)
 
-    def zoom_change(self, step=5):
-        """Modifies the zoom level"""
-        self.zoom_set(self.zoom_level + step)
-
-    def zoom_in(self, widget=None):
-        """Increases the zoom level by 5 steps"""
-        self.zoom_change(5)
-
-    def zoom_out(self, widget=None, step=5):
-        """Reduces the zoom level by 5 steps"""
-        self.zoom_change(-5)
+    def zoom_change(self, action, step, unknown):
+        """ Action handle for zoom change """
+        self.zoom_set(self.zoom_level + step.get_int32())
 
     def get_file_path_from_dnd_dropped_uri(self, uri):
         """Extracts the path from an uri"""
@@ -1044,15 +1118,9 @@ class PdfArranger:
             path = path[5:]  # 5 is len('file:')
         return path
 
-    def rotate_page_right(self, widget, data=None):
-        self.rotate_page(90)
-
-    def rotate_page_left(self, widget, data=None):
-        self.rotate_page(-90)
-
-    def rotate_page(self, angle):
+    def rotate_page(self, action, angle, unknown):
         """Rotates the selected page in the IconView"""
-
+        angle = angle.get_int32()
         model = self.iconview.get_model()
         selection = self.iconview.get_selected_items()
         if len(selection) > 0:
@@ -1073,9 +1141,8 @@ class PdfArranger:
                 new_angle = new_angle % 360
                 model.set_value(iter, 6, new_angle)
                 self.update_geometry(iter)
-        self.reset_iv_width()
 
-    def crop_page_dialog(self, widget):
+    def crop_page_dialog(self, action, parameter, unknown):
         """Opens a dialog box to define margins for page cropping"""
 
         sides = ('L', 'R', 'T', 'B')
@@ -1103,36 +1170,36 @@ class PdfArranger:
                             flags=Gtk.DialogFlags.MODAL,
                             buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                      Gtk.STOCK_OK, Gtk.ResponseType.OK))
-        dialog.set_size_request(340, 250)
         dialog.set_default_response(Gtk.ResponseType.OK)
-
+        margin = 12
+        label= Gtk.Label(label=_('Cropping does not remove any content\nfrom the PDF file, it only hides it.'))
+        dialog.vbox.pack_start(label, False, False, 0)
         frame = Gtk.Frame(label=_('Crop Margins'))
-        dialog.vbox.pack_start(frame, False, False, 20)
-
-        vbox = Gtk.VBox(False, 0)
-        frame.add(vbox)
+        frame.props.margin = margin
+        dialog.vbox.pack_start(frame, True, True, 0)
+        grid = Gtk.Grid()
+        grid.set_column_spacing(margin)
+        grid.set_row_spacing(margin)
+        grid.props.margin = margin
+        frame.add(grid)
 
         spin_list = []
-        units = 2 * [_('%% of width').replace('%%','%')] +\
-                2 * [_('%% of height').replace('%%','%')]
-        for side in sides:
-            hbox = Gtk.HBox(True, 0)
-            vbox.pack_start(hbox, False, False, 5)
-
+        units = 2 * [_('% of width')] + 2 * [_('% of height')]
+        for row, side in enumerate(sides):
             label = Gtk.Label(label=side_names[side])
-            label.set_alignment(0, 0.0)
-            hbox.pack_start(label, True, True, 20)
+            label.set_alignment(0, 0)
+            grid.attach(label, 0, row, 1, 1)
 
             adj = Gtk.Adjustment(100. * crop.pop(0), 0.0, 99.0, 1.0, 5.0, 0.0)
             spin = Gtk.SpinButton(adjustment=adj, climb_rate=0, digits=1)
             spin.set_activates_default(True)
             spin.connect('value-changed', set_crop_value, side)
             spin_list.append(spin)
-            hbox.pack_start(spin, False, False, 30)
+            grid.attach(spin, 1, row, 1, 1)
 
             label = Gtk.Label(label=units.pop(0))
-            label.set_alignment(0, 0.0)
-            hbox.pack_start(label, True, True, 0)
+            label.set_alignment(0, 0)
+            grid.attach(label, 2, row, 1, 1)
 
         dialog.show_all()
         result = dialog.run()
@@ -1150,9 +1217,6 @@ class PdfArranger:
                 self.update_geometry(pos)
             if modified:
                 self.set_unsaved(True)
-            self.reset_iv_width()
-        elif result == Gtk.ResponseType.CANCEL:
-            print(_('Dialog closed'))
         dialog.destroy()
 
     def reverse_order_available(self, selection):
@@ -1173,7 +1237,7 @@ class PdfArranger:
 
         return True
 
-    def reverse_order(self, widget, data=None):
+    def reverse_order(self, action, parameter, unknown):
         """Reverses the selected elements in the IconView"""
 
         model = self.iconview.get_model()
@@ -1215,10 +1279,9 @@ class PdfArranger:
         about_dialog.connect('delete_event', lambda w, *args: w.destroy())
         about_dialog.show_all()
 
-    def error_message_dialog(self, msg):
+    def error_message_dialog(self, msg, msg_type=Gtk.MessageType.ERROR):
         error_msg_dlg = Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL,
-                                          type=Gtk.MessageType.ERROR,
-                                          parent=self.window,
+                                          type=msg_type, parent=self.window,
                                           message_format=str(msg),
                                           buttons=Gtk.ButtonsType.OK)
         response = error_msg_dlg.run()
@@ -1254,7 +1317,7 @@ class PDFDoc:
 
 class PDFRenderer(threading.Thread, GObject.GObject):
 
-    def __init__(self, model, pdfqueue, resample=1.):
+    def __init__(self, model, pdfqueue, resample):
         threading.Thread.__init__(self)
         GObject.GObject.__init__(self)
         self.model = model
@@ -1266,30 +1329,24 @@ class PDFRenderer(threading.Thread, GObject.GObject):
         for idx, row in enumerate(self.model):
             if self.quit:
                 return
-            if not row[1]:
-                try:
-                    nfile = row[2]
-                    npage = row[3]
-                    pdfdoc = self.pdfqueue[nfile - 1]
-                    page = pdfdoc.document.get_page(npage - 1)
-                    w, h = page.get_size()
-                    thumbnail = cairo.ImageSurface(cairo.FORMAT_ARGB32,
-                                                   int(w / self.resample),
-                                                   int(h / self.resample))
-                    cr = cairo.Context(thumbnail)
-                    if self.resample != 1.:
-                        cr.scale(1. / self.resample, 1. / self.resample)
-                    page.render(cr)
-                    time.sleep(0.003)
-                    GObject.idle_add(self.emit, 'update_thumbnail',
-                                     idx, thumbnail, self.resample,
-                                     priority=GObject.PRIORITY_LOW)
-                except Exception as e:
-                    print(e)
-
+            try:
+                nfile = row[2]
+                npage = row[3]
+                pdfdoc = self.pdfqueue[nfile - 1]
+                page = pdfdoc.document.get_page(npage - 1)
+                w, h = page.get_size()
+                thumbnail = cairo.ImageSurface(cairo.FORMAT_ARGB32,
+                                               int(w / self.resample),
+                                               int(h / self.resample))
+                cr = cairo.Context(thumbnail)
+                if self.resample != 1.:
+                    cr.scale(1. / self.resample, 1. / self.resample)
+                page.render(cr)
+                GObject.idle_add(self.emit, 'update_thumbnail',
+                                 idx, thumbnail, self.resample,
+                                 priority=GObject.PRIORITY_LOW)
+            except Exception as e:
+                traceback.print_exc()
 
 def main():
-    """This function starts pdfarranger"""
-    GObject.threads_init()
-    PdfArranger()
-    Gtk.main()
+    PdfArranger().run(sys.argv)
